from __future__ import print_function, unicode_literals
from random import choice
from PyInquirer import style_from_dict, Token, prompt
from PyInquirer import Validator, ValidationError
import regex
from . import ifinal
from . import dummy
import re
import sys


style = style_from_dict({
    Token.QuestionMark: '#E91E63 bold',
    Token.Selected: '#8B64CE bold',
    Token.Instruction: '#EBF2FA',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '#00ff00 bold',
})

# TODO check title option


class RangeValidator(Validator):
    def validate(self, document):

        ok = re.match('^\d+-\d+$', document.text)
        if ok:
            start = re.findall('^\d+(?=-)', document.text)[0]
            end = int(re.findall('(?<=-)\d+', document.text)[0])
            vaild_start = int(start) > 0 and int(start) < int(playlist_len - 1)
            vaild_end = int(end) > int(start) and int(end) <= (playlist_len)
            print(start)
            print(end)
            if not (ok and vaild_start and vaild_end):
                raise ValidationError(
                    message=f"Please enter a range between 1-{playlist_len}",
                    cursor_position=len(document.text))  # Move cursor to end
        else:
            raise ValidationError(
                message=f"Please enter a range between 1-{playlist_len}",
                cursor_position=len(document.text))  # Move cursor to end


class IndexValidator(Validator):
    def validate(self, document):
        user_values = re.findall(',?\d+,?', str(document.text))
        index = re.findall('\d+', str(document.text))
        other = re.findall('[^,\d\s]+', str(document.text))
        if not user_values:
            raise ValidationError(message=f"Please enter valid index (ex 1 or 1,2) (index < {playlist_len})",
                                  cursor_position=len(str(document.text)))  # Move cursor to end

        if other and user_values:
            raise ValidationError(
                message=f"Please enter valid index (ex 1 or 1,2) (index < {playlist_len})",
                cursor_position=len(str(document.text)))  # Move cursor to end

        elif index and user_values:
            greater_index = ""
            for i in index:
                if int(i) > playlist_len or int(i) == 0:
                    greater_index = greater_index + str(i)

                    raise ValidationError(
                        message=f"Please enter valid index ex 1 or 1,2 (index < {playlist_len})",
                        cursor_position=len(str(document.text)))  # Move cursor to end
                    break


def link_type(url):
    video = regex.search(r'v=[0-9a-zA-Z]*', url)
    playlist = regex.search(r'list=[0-9a-zA-Z\-_]*', url)

    if playlist:
        type = 'playlist'
        return type
    elif video:
        type = 'video'
        return type


def choices_video_quality(answers):
    choices = []
    # print(urli
    # )
    if urlis == 'video':
        results = ifinal.option_with_filesize(link)
        if not results:
            sys.exit(1)
        choices = results['only_video']
        choices = choices
    else:
        choices = [{'name': '1080p HD', 'value': '137'}, {'name': '720p HD', 'value': '136'}, {'name': '480p', 'value': '135'}, {
            'name': '360p', 'value': '134'}, {'name': '240p', 'value': '133'}, {'name': '144p', 'value': '132'}]
    return choices


def choices_video_audio_quality(vid_aud_quality_ans):
    choices = []
    if urlis == 'video':
        results = ifinal.option_with_filesize(link)
        if not results:
            sys.exit(1)
        choices = results['video_audio']
        choices = choices
    else:
        choices = [{'name': '1080p HD', 'value': '137+bestaudio'}, {'name': '720p HD', 'value': '22/136+bestaudio'}, {'name': '480p', 'value': '135+bestaudio'}, {
            'name': '360p', 'value': '18/134+bestaudio'}, {'name': '240p', 'value': '133+bestaudio'}, {'name': '144p', 'value': '132+bestaudio/'}]
    return choices


def playlist():
    playlist_option_ques = [
        {
            'type': 'list',
            'name': 'option',
            'message': 'Choose option: ',
            'choices': [
                {
                    'name': 'Whole Playlist',
                    'value': 'all'
                },
                {
                    'name': 'Single Video (ex 1) or Specify Videos (ex 1,2)',
                    'value': 'specific'
                },
                {
                    'name': 'Range (ex 1-2)',
                    'value': 'range'
                },

            ],

        },
        {
            'type': 'input',
            'name': 'playlist_items',
            'message': 'Enter range (ex 1-2):',
            'validate': RangeValidator,
            'when': lambda answers: answers['option'] == 'range'
        },
        {
            'type': 'input',
            'name': 'playlist_items',
            'message': 'Enter Video index (ex 1,2):',
            'validate': IndexValidator,
            'when': lambda answers: answers['option'] == 'specific'
        }
    ]
    playlist_option_ans = prompt(playlist_option_ques, style=style)
    if not playlist_option_ans:
        sys.exit(1)
    video_answers = video()
    post_answers = post_playlist_questions()
    final_type = {'type': 'playlist'}
    return(dict(final_type, **playlist_option_ans, **video_answers, **post_answers))


def video_audio_quality():
    vid_aud_quality_ques = {
        'type': 'list',
        'name': 'format',
        'message': 'Choose Quality',
        'choices': choices_video_audio_quality,
        # 'when': lambda answers: answers['want'] == 'video_audio'
    }
    vid_aud_quality_ans = prompt(vid_aud_quality_ques, style=style)
    if not vid_aud_quality_ans:
        sys.exit(1)
    return vid_aud_quality_ans


def video_quality():
    video_quality_que = {
        'type': 'list',
        'name': 'format',
        'message': 'Choose Quality',
        'choices': choices_video_quality,
        # 'when': lambda answers: answers['want'] == 'video'
    }
    vid_quality_ans = prompt(video_quality_que, style=style)
    if not vid_quality_ans:
        sys.exit(1)
    return vid_quality_ans


def audio_quality():
    audio_quality_que = {
        'type': 'list',
        'name': 'audio-quality',
        'message': 'Choose Quality',
        'choices': [
                {
                    'name': 'Mp3 (320kbps)',
                    'value': '320'
                },
            {
                    'name': 'Mp3 (256kbps)',
                    'value': '256'
            },
            {
                    'name': 'Mp3 (192kbps)',
                    'value': '192'
            },
            {
                    'name': 'Mp3 (156kbps)',
                    'value': '156',
            }
        ],
        # 'when': lambda answers: answers['want'] == 'audio'
    }

    audio_quality_ans = {
        'format': 'bestaudio',
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec': 'mp3',
            'preferredquality': '',
            #'embed-thumbnail': True
        }]
    }
    a = prompt(audio_quality_que, style=style)
    if not a:
        sys.exit(1)
    audio_quality_ans['postprocessors'][0]['preferredquality'] = a.get(
        'audio-quality')
    return audio_quality_ans


def video():
    video_want_ques = {
        'type': 'list',
        'name': 'want',
        'message': 'What do you want?',
        'choices': [
                {
                    'name': 'Video + Audio',
                    'value': 'video_audio'
                },
            {
                    'name': 'Audio only',
                    'value': 'audio'
            },
            {
                    'name': 'Video (Really! Without audio)',
                    'value': 'video'
            }
        ]
    }
    video_want_answers = prompt(video_want_ques, style=style)
    if not video_want_answers:
        sys.exit(1)
    if video_want_answers['want'] == 'video_audio':
        answer = video_audio_quality()
        return(dict(video_want_answers, **answer))
    elif video_want_answers['want'] == 'video':
        answer = video_quality()
        return(dict(video_want_answers, **answer))
    elif video_want_answers['want'] == 'audio':
        answer = audio_quality()
        return(dict(video_want_answers, **answer))
    # return video_want_answers


def post_playlist_questions():
    post_playlist_que = [
        {
            'type': 'confirm',
            'name': 'output1',
            'message': 'Want Video Indexing?',
            'default': False
        },
        {
            'type': 'confirm',
            'name': 'output2',
            'message': 'Playlist videos in separate directory?',
            'default': False
        }
    ]
    post_playlist_ans = prompt(post_playlist_que, style=style)
    if not post_playlist_ans:
        sys.exit(1)
    if post_playlist_ans['output1'] == True and post_playlist_ans['output2'] == True:
        output = {
            'outtmpl': "%(playlist)s/%(playlist_index)s. %(title)s.%(ext)s"}
        return output
    elif post_playlist_ans['output1'] == False and post_playlist_ans['output2'] == True:
        output = {'outtmpl': "%(playlist)s/%(title)s.%(ext)s"}
        return output
    elif post_playlist_ans['output1'] == False and post_playlist_ans['output2'] == False:
        output = {'outtmpl': "%(title)s.%(ext)s"}
        return output
    elif post_playlist_ans['output1'] == True and post_playlist_ans['output2'] == False:
        output = {'outtmpl': "%(playlist_index)s. %(title)s.%(ext)s"}
        return output


def clear_noise(output):
    entries = ('type', 'option', 'want')
    for key in entries:
        if key in output:
            del output[key]
    return output


def opts(url):
    global link
    global urlis
    link = url
    # print(link)

    validation = link_type(url)
    if validation == 'playlist':
        urlis = 'playlist'
        global playlist_len
        playlist_len = dummy.playlist_length(url)
        options = playlist()
        return clear_noise(options)

    elif validation == 'video':
        urlis = 'video'
        # print(urlis)
        options = video()
        return clear_noise(options)
    else:

        # urlis = 'playlist'  not sure about it
        options = video()
        return clear_noise(options)
    # print(urlis)



import pyperclip
import regex
from rich import print
from youtubesearchpython import SearchVideos, SearchPlaylists
from PyInquirer import style_from_dict, Token, prompt
from PyInquirer import Validator, ValidationError
from halo import Halo
import re
import sys


spinner = Halo(text='Searching', spinner='dots')

style = style_from_dict({
    Token.QuestionMark: '#E91E63 bold',
    Token.Selected: '#8B64CE bold',
    Token.Instruction: '#EBF2FA',  # default
    Token.Answer: '#2196f3 bold',
    Token.Question: '#00ff00 bold',
})


# TODO search_video(query) in validator | and create two seprate vaildator of video query and playlist query


class queryValidator(Validator):
    def validate(self, document):
        query = document.text
        if len(query) >= 55:
            raise ValidationError(
                message='Please be concise. Query should be less than 50 characters.',
                cursor_position=len(document.text))  # Move cursor to end
        elif query == "":
            raise ValidationError(
                message='Please enter something.',
                cursor_position=len(document.text))  # Move cursor to end


class linkValidator(Validator):
    def validate(self, document):
        url = document.text
        if urlvalidator(url):
            return True
        else:
            raise ValidationError(
                message='Please enter a valid youtube link',
                cursor_position=len(document.text))  # Move cursor to end


def urlvalidator(url):
    if regex.match(r'^(http(s)?:\/\/)?((w){3}.)?youtu(be|.be)?(\.com)?\/.+', url):
        return True
    else:
        return False


def check_clipboard():
    clipboard_content = pyperclip.paste()
    if urlvalidator(clipboard_content):
        return True
    else:
        return False


def search_query():
    questions = [
        {
            'type': 'input',
            'name': 'query',
            'message': 'Search:',
            'validate': queryValidator
        }]
    answers = prompt(questions, style=style)
    if not answers:
        print("baba")
        
    return answers



def search_video(query):
    spinner.start()

    data = SearchVideos(query, offset=1, mode="dict", max_results=8)
    final_data = data.result()
    if not final_data:
        spinner.stop()

        print("[bold red]  Something went wrong. Try another query or check network connenctivity[bold red]")
        
    else:
        options = []
        for i in final_data['search_result']:
            temp_dict = {}
            temp_dict.update({'name': i['title'], 'value': i['link']})
            options.append(temp_dict)
        spinner.stop()
    

        return options
 



def search_playlist(query):
    spinner.start()

    data = SearchPlaylists(query, offset=1, mode='dict', max_results=8)
    final_data = data.result()
    if not final_data:
        spinner.stop()

        print("[bold red]  Something went wrong. Try another query or check network connenctivity[/bold red]")
        
    else:   
        options = []
        for i in final_data['search_result']:
            temp_dict = {}
            temp_dict.update({'name': i['title'], 'value': i['link']})
            options.append(temp_dict)
        spinner.stop()
    
        return options


def get_options(answers):
    options = [
        {
            'name': 'Enter URL',
            'value': 'manual'
        },
        {
            'name': 'Search Video',
            'value': 'video_search'
        },
        {
            'name': 'Search Playlist',
            'value': 'playlist_search'
        }
    ]

    if check_clipboard():
        options[0] = {
            'name': 'Enter other URL',
            'value': 'manual'
        }
        options.insert(0, {
            'name': 'URL form clipboard',
            'value': 'clipboard',
        })
    return options


def start_1():
    ques1 = [
        {
            'type': 'list',
            'name': 'source',
            'message': 'Select:',
            'choices': get_options
        },
        {
            'type': 'input',
            'name': 'url',
            'message': 'Enter Link:',
            'validate': linkValidator,
            'when': lambda ans1: ans1['source'] == 'manual'
        },
        {
            'type': 'input',
            'name': 'query',
            'message': 'Search:',
            'validate': queryValidator,
            'when': lambda ans1: ans1['source'] == 'video_search' or ans1['source'] == 'playlist_search'

        }]
    ans1 = prompt(ques1, style=style)
    if not ans1:
        print('hello')
         

    if ans1['source'] == 'video_search':
        query = ans1['query']
        ques2 = {
            'type': 'list',
            'name': 'url',
            'message': 'Results (select):',
            'choices': search_video(query)
        }
        ans2 = prompt(ques2, style=style)
        if not ans2:
            print("hi")
            
        return(ans2['url'])

    elif ans1['source'] == 'playlist_search':
        query = ans1['query']
        ques2 = {
            'type': 'list',
            'name': 'url',
            'message': 'Results (select):',
            'choices': search_playlist(query)
        }
        ans2 = prompt(ques2, style=style)
        if not ans2:
            print("mama")
            
        return(ans2['url'])

    elif ans1['source'] == 'manual':
        ans2 = {}
        ans2['url'] = ans1['url']
        return(ans2['url'])

    elif ans1['source'] == 'clipboard':
        ans2 = {}
        ans2['url'] = pyperclip.paste()
        return(ans2['url'])



from __future__ import unicode_literals
import youtube_dl
import humanfriendly
import re
from halo import Halo
from rich import print
import sys


video_quality_list = {
    '18': '360p',
    '22': '720p',
    '37': '1080p',
    '38': '4K',
    '160': '144p',
    '133': '240p',
    '134': '360p',
    '135': '480p',
    '136': '720p',
    '298': '720p 60fps',
    '137': '1080p',
    '299': '1080p 60fps',
    '264': '1440p',
    '138': '2160p',
    '266': '2160p 60fps',
    '82': '360p',
    '83': '480p',
    '84': '720p',
    '85': '1080p',
    '394': '144p',
    '395': '240p',
    '396': '360p',
    '397': '480p',
    '398': '720p',
    '399': '1080p',
    '400': '1440p',
    '401': '2160p',
    '402': '2880p',
    # webm below
    '242': '240p',
    '243': '360p',
    '244': '480p',
    '247': '720p',
    '302': '720p 60fps',
    '248': '1080p',
    '303': '1080p 60fps',
    '271': '1440p',
    '308': '1440p 60fps',
    '313': '2160p',
    '315': '2160p 60fps',
    '272': '4320p (DASH Video)',
    '100': '360p (3D) ',
    '101': '480p (3D) ',
    '102': '720p (3D) ',
    '330': '144p 60fps (HDR)',
    '331': '240p 60fps (HDR)',
    '332': '360p 60fps (HDR)',
    '333': '480p 60fps (HDR)',
    '334': '720p 60fps (HDR)',
    '335': '1080p 60fps (HDR)',
    '336': '1440p 60fps (HDR)',
    '337': '2160p 60fps (HDR)',
    '219': '144p',
    '278': '144p',
    '167': '360p',
    '168': '480p',
    '218': '480p',
    '245': '480p',
    '246': '480p',
    '169': '1080p',
}


class MyLogger(object):
    def debug(self, msg):
        pass

    def error(self, msg):
        pass

    def warning(self, msg):
        pass


def option_with_filesize(url):
    spinner = Halo(text='Fetching available video qualities.', spinner='dots')
    spinner.start()
    options = {}  # return variable
    ydl_opts = {'logger': MyLogger()}  # youtube-dl options
    video_audio, only_video = [], []  # to store video size quality and format_id
    audio = {}  # to store audio filesize
    ydl = youtube_dl.YoutubeDL(ydl_opts)
    meta = None
    try:
        meta = ydl.extract_info(url, download=False)
    except:
        spinner.stop()
        print("[bold red]  Something went wrong.[/bold red]")
        sys.exit(1)
    

    for i in meta['formats']:
        # print(i)

        readable_size = None
        new_format_id = None
        video_size = None
        audio_size = None
        video_quality = None

        # only greater than 1080p and video only
        if i['ext'] == 'webm' and re.findall('(271|272|308|313|315|336|337)', i['format_id']):
            # print("asddddddddddddddddddddddddddddddddddddddddddddddddddddddfdsdaaaaaaaaaaaaa")
            video_quality = video_quality_list[i['format_id']]
            format_id = i['format_id']
            if i['filesize'] == None:  # kaka
                readable_size = ""
                readable_size_without_audio = ""
                new_format_id = f"{i['format_id']}+{list(audio.keys())[-1]}"
                video_size = 0
            else:
                video_size = i['filesize']
                audio_size = list(audio.values())[-1]
                final_size = video_size + audio_size
                readable_size = f"({humanfriendly.format_size(final_size, binary=True)})"
                readable_size_without_audio = f"({humanfriendly.format_size(int(i['filesize']), binary=True)})"
                new_format_id = f"{i['format_id']}+{list(audio.keys())[-1]}"
            video_audio.append(
                {'name': f"{video_quality} {readable_size}", 'value': new_format_id})
            only_video.append(
                {'name': f"{video_quality} {readable_size_without_audio}", 'value': format_id})

        # if i['ext'] == 'mp4' and re.findall('avc1', i['vcodec']) and re.findall('(38|133|134|135|136|137|138|160|212|213|214|215|216|217|264|266|298|299)',i['format_id']): #mp4 video only
        # mp4 video only
        if i['ext'] == 'mp4' and re.findall('(38|133|134|135|136|137|138|160|212|213|214|215|216|217|264|266|298|299)', i['format_id']):

            video_quality = video_quality_list[i['format_id']]
            audio_size = list(audio.values())[-1]
            # filesize in human friendly format (video + audio)
            readable_size = None
            new_format_id = None  # itag (video+audio)
            video_size = None
            audio_size = None
            format_id = i['format_id']  # itag
            # filesize in human friendly format (only video)
            readable_size_without_audio = None

            if re.findall('38', str(i['format'])):
                if i['filesize'] == None:  # baba
                    # print("baba")
                    readable_size = ""
                    new_format_id = i['format_id']
                    video_size = 0
                    readable_size_without_audio = ""
                else:  # mama
                    # print("mama")
                    readable_size = f"({humanfriendly.format_size(i['filesize'], binary=True)})"
                    readable_size_without_audio = f"({humanfriendly.format_size(int(i['filesize']), binary=True)})"
                    new_format_id = i['format_id']
                    video_size = i['filesize']
            else:
                video_quality = video_quality_list[i['format_id']]
                if i['filesize'] == None:  # kaka
                    # print("kaka")
                    readable_size = ""
                    readable_size_without_audio = ""
                    new_format_id = f"{i['format_id']}+{list(audio.keys())[-1]}"
                    video_size = 0
                else:  # dada
                    # print("dada")
                    video_size = i['filesize']
                    audio_size = list(audio.values())[-1]
                    final_size = video_size + audio_size
                    readable_size = f"({humanfriendly.format_size(final_size, binary=True)})"
                    readable_size_without_audio = f"({humanfriendly.format_size(int(i['filesize']), binary=True)})"
                    new_format_id = f"{i['format_id']}+{list(audio.keys())[-1]}"
            # itag 18,22 contain video+audio in mp4 format also filesize is large and sometimes
            # filesize also not availabe.
            # video_audio.append(
            #    {'name': f"{video_quality} {readable_size}", 'value': new_format_id})
            # only_video.append(
            #    {'name': f"{video_quality} {readable_size_without_audio}", 'value': format_id})
            if not re.findall('(18|22|37)', str(new_format_id)):
                video_audio.append(
                    {'name': f"{video_quality} {readable_size}", 'value': new_format_id})
                only_video.append(
                    {'name': f"{video_quality} {readable_size_without_audio}", 'value': format_id})

        # |600|249|250|251|171|172
        elif re.findall('(139|140|141)', str(i['format_id'])):
            audio[i['format_id']] = i['filesize']
            # if re.findall('(139|140|141)', str(i['format_id'])):
            #    audio['m4a'] = { i['format_id']: i['filesize']}
            # else:
            #    audio['webm'] = { i['format_id']: i['filesize']}
    options['video_audio'] = video_audio
    options['only_video'] = only_video
    # print(audio)
    spinner.stop()
    # print(options)
    # print(f"  [bold green]Title: [/bold green][bold #2196f3]{meta['title']}[/bold #2196f3]")
    return options

#
#url = "https://www.youtube.com/watch?v=xcJtL7QggTI"
#try:
#    print(option_with_filesize(url))
#except Exception as e:
#    print(e)
#print(option_with_filesize(url))


from __future__ import unicode_literals
from re import findall
import pyfiglet
from . import iy
from halo import Halo
import time
import sys
import click


import re
from . import questions
import youtube_dl
from rich.progress import Progress

from rich import print
from rich.progress import (
    BarColumn,
    DownloadColumn,
    TextColumn,
    TransferSpeedColumn,
    TimeRemainingColumn,
    Progress,
    TaskID,
)


#
# TODO regerous testing and also add doc

progress = Progress(
    TextColumn("[bold blue]{task.description}\n", justify="right"),
    BarColumn(),
    "[progress.percentage]{task.percentage:>3.1f}%",
    "•",
    DownloadColumn(),
    "•",
    TransferSpeedColumn(),
    "•",
    TimeRemainingColumn(),
    transient=True

)

# FIXME inbetween error when one module error cause another tho run

global current_deleting_file
current_deleting_file = ""
class MyLogger(object):
    def error(self, msg):
        if re.findall("requested format not available", msg):
            print(f"[bold red]ERROR: Requested format not available[/bold red]")
        else:
            print(f"[bold red]{msg}[/bold red]")

    def debug(self, msg):

        #
        # print(msg)
        if re.findall("has already been downloaded", msg):
            print(f" [bold #ADFF2F]{msg}[/bold #ADFF2F]")
        # FIXME if not ffmpeg then
        a = re.findall('\[ffmpeg\].*', msg)
        if a != []:
            if re.findall("Correcting container", a[0]):
                progress.update(0, visible=False)
                print(f"[bold #00ff00] {a[0]}[/bold #00ff00]")
            elif re.findall("Merging formats", a[0]):
                progress.update(0, visible=False)
                print(
                    f"[bold #ADFF2F]  Done downloading, now converting ...[/bold #ADFF2F]")
                print(f"[bold #00ff00]✔{a[0]}[/bold #00ff00]")
            elif re.findall("Destination:", a[0]):
                progress.update(0, visible=False)
                print(
                    f"[bold #ADFF2F]  Done downloading, now converting ...[/bold #ADFF2F]")
                print(f"[bold #00ff00]✔{a[0]}[/bold #00ff00]")
        if re.findall("Deleting original", msg):
            #current_deleting_file = msg
            #print(current_deleting_file)
            #print(f"[bold #00ff00]✔ Done converting[/bold #00ff00]")
            if current_deleting_file == None:
                print(f"[bold #00ff00]✔ Done converting[/bold #00ff00]")
            current_deleting_file = msg
            if current_deleting_file == msg:
                current_deleting_file = None

    def warning(self, msg):
        print(f"  [bold #FF8C00]{msg}[/bold #FF8C00]")


def my_hook(d):
    # print(d)
    total_bytes = None
    if "total_bytes_estimate" in d:
        total_bytes = d['total_bytes_estimate']
    elif "total_bytes" in d:
        total_bytes = d['total_bytes']

    if d['status'] == 'finished':

        # print(d)
        #progress.update(0, visible=False)
        filename = d['filename']
        slim_filename = re.sub('-[a-zA-Z0-9]+.f[1-9]+', '', filename)

        if "_elapsed_str" in d and re.findall('f\d', filename):
            if re.findall('f[1-3](0|1|3|4|6|7|9)[0-9]', filename) or re.findall('(\.mp4)', filename):
                print(
                    f"  [bold #ADFF2F	]Downloaded: [Video] {slim_filename}[/bold #ADFF2F	] [bold blue]100% of {d['_total_bytes_str']} in {d['_elapsed_str']}")

            elif re.findall('f(139|140|141|600|249|250|251|171|172)', filename) or re.findall('(\.m4a)', filename):
                print(
                    f"  [bold #ADFF2F	]Downloaded: [Audio] {slim_filename}[/bold #ADFF2F	] [bold blue]100% of {d['_total_bytes_str']} in {d['_elapsed_str']}")

            elif re.findall('f[1-3](2|7|8)', filename):
                print(
                    f"  [bold #ADFF2F	]Downloaded: [Video+Audio] {slim_filename}[/bold #ADFF2F	] [bold blue]100% of {d['_total_bytes_str']} in {d['_elapsed_str']}")

        else:
            print(
                f"  [bold #ADFF2F	]Downloaded: {slim_filename}[/bold #ADFF2F	]")

    if d['status'] == 'downloading':
        downloader_bytes = d['downloaded_bytes']
        filename = d['filename']
        slim_filename = re.sub('-[a-zA-Z0-9]+.f[1-9]+', '', filename)

        if re.findall('f[1-3](0|1|3|4|6|7|9)[0-9]', filename):
            progress.update(0, completed=downloader_bytes, total=total_bytes,
                            description=f"  [Vidio] {slim_filename}", visible=True)
        elif re.findall('f(139|140|141|600|249|250|251|171|172)', filename):
            progress.update(0, completed=downloader_bytes, total=total_bytes,
                            description=f"  [Audio] {slim_filename}", visible=True)
        elif re.findall('f(18|22|37|38)', filename):
            progress.update(0, completed=downloader_bytes, total=total_bytes,
                            description=f"  [Vidio+Audio] {slim_filename}", visible=True)
        else:
            progress.update(0, completed=downloader_bytes, total=total_bytes,
                            description=f"  {slim_filename}", visible=True)


@click.command()
def run():
    """
    Simple interactive youtube downloader written in python. Interactively select the quality and format for youtube-dl

    Run you-dl thats all you need. :-)
    """

    result = pyfiglet.figlet_format("You-dl", font="slant")
    default_opts = {
        'outtmpl': '%(title)s.%(ext)s',
        'progress_hooks': [my_hook],
        'verbose': False,
        'http_chunk_size': 2097152000,
        'logger': MyLogger(),

        #'merge_output_format': 'mp4',
    }
    print(f"[bold red]{result}[/bold red]")
    print(f"[bold #fec601]Hi, Welcome to You-dl[/bold #fec601]")
    url = None
    try:
        url = questions.start_1()
    except Exception as e:
        print(e)
        sys.exit(1)
    user_opts = iy.opts(url)
    #print(user_opts)
    if not user_opts:
        print("[bold red]Use keyboard for selection[/bold red]")
        sys.exit(1)
        
    ydl_opts = dict(default_opts, **user_opts)
    # print(ydl_opts)
    try:
        with youtube_dl.YoutubeDL(ydl_opts) as ydl:
            with progress:
                time.sleep(0.3)
                task_id = progress.add_task("Loading...", start=False)
                progress.start_task(0)
                ydl.download([url])
    except:
        pass

def main():
    try: 
        run()

    except KeyboardInterrupt:
        print("it happen")
        sys.exit(1)
    except TypeError:
        #print("[bold red]Check network connection and try again.[/bold red]")
        sys.exit(1)
    except KeyError:
        print("[bold red]Use keyboard for selection[/bold red]")
        sys.exit(1)
    except ValueError:
        print("[bold red]Use keyboard for selection[/bold red]")
        sys.exit(1)
    except Exception as e:
        print(f"[red]{e}[/red]")
        sys.exit(1)


main()

# type error when net is of and Index error when invalid query
# song_name Please try again with a different keyword.



from __future__ import unicode_literals
import youtube_dl
import humanfriendly
import re
from rich import print

def video_
video = []
ydl_opts, audio = {}
ydl = youtube_dl.YoutubeDL(ydl_opts)
meta = ydl.extract_info(url, download=False)

for i in meta['formats']:
    readable_size = None
    new_format_id = None
    video_size = None
    audio_size = None
    if i['ext'] == 'mp4' and re.findall('avc1', i['vcodec']):
        audio_size = list(audio.values())[-1]
        readable_size = None
        new_format_id = None
        video_size = None
        audio_size = None
        if re.findall('^(18|22|37|38)', str(i['format'])):
            if i['filesize'] == None:
                readable_size = "Not Available"
                new_format_id = i['format_id']
                video_size = 0
            else:
                readable_size = humanfriendly.format_size(
                    i['filesize'], binary=True)
                new_format_id = i['format_id']
                video_size = i['filesize']
        else:
            if i['filesize'] == None:
                readable_size = "Not Available"
                new_format_id = i['format_id']
                video_size = 0
            else:
                video_size = i['filesize']
                audio_size = list(audio.values())[-1]
                print(video_size)
                final_size = video_size + audio_size
                readable_size = humanfriendly.format_size(
                    final_size, binary=True)
                new_format_id = f"{i['format_id']}+{list(audio.keys())[-1]}"
        video.append(
            {'name': f"{i['format_note']} ({readable_size})", 'value': new_format_id})
    elif i['format_note'] == 'tiny':
        audio[i['format_id']] = i['filesize']
print(video)
print(audio)



from iy.version import __version__
try:
    from setuptools import setup, find_namespace_packages
except ModuleNotFoundError as err:
    print(err.msg)
    exit(1)


with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="you-dl",
    version=__version__,
    author="vishal dongre",
    author_email="xvishaldongre@gmail.com",
    description="A simple interactive youtube downloader written in python. Interactively select the quality and format for youtube-dl",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xvishaldongre/you-dl",
    packages=find_namespace_packages(),
    install_requires=['youtube-dl', 'prompt_toolkit==1.0.14','PyInquirer',
                      'regex', 'youtube-search-python', 'pyperclip',
                      'pyfiglet', 'halo', 'click', 'rich', 'humanfriendly'],
    include_package_data=True,
    
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Environment :: Console",
        "Topic :: Multimedia ",
    ],
    license='GPLv3',
    python_requires='>=3.5',
    entry_points={
        'console_scripts': [
            'you-dl = iy.yt:main'
        ]}
)



import pafy
from rich import print
from halo import Halo

def playlist_length(url):
    spinner = Halo(text='Fetching playlist metadata.', spinner='dots')
    spinner.start()
    playlist = None
    try:
        playlist = pafy.get_playlist(url)
    except:
        spinner.stop()
        print("[red bold]  Unable to get Playlist length. Please check your network connectivity and try again.")    
    global playlist_length
    playlist_length = len(playlist['items'])
    spinner.stop()

    return playlist_length
